package com.incubator.game.player;

import com.incubator.game.GGame;
import com.incubator.game.data.po.PlayerInfoPO;
import com.incubator.game.util.Const.PlayerEventType;
import com.incubator.game.util.Const.ManagerType;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 玩家实体
 */
public class Player extends GPlayer {

    /** 玩家绑定的所有功能管理类 **/
    public Map<ManagerType, ModuleManager> allManagers = new HashMap<>();

    /** 玩家数据 **/
    public PlayerInfoPO data;

    public Player(PlayerInfoPO playerData) {
        try {
            this.init(playerData);
        } catch (Exception e) {
            logger.error("玩家初始化数据出错...{}", e.getMessage());
        }
    }

    /**
     * 注册玩家管理类
     *
     * @param manager
     */
    private void registerManager(ModuleManager manager) {
        if (manager.getManagerType() == null) {
            logger.error("{}, manager.getManagerType 为空", manager.getClass());
        }
        this.allManagers.put(manager.getManagerType(), manager);
    }

    /**
     * 初始化玩家数据，注册绑定玩家各功能管理类
     *
     * @param data
     */
    private void init(PlayerInfoPO data) {
        this.data = data;
    }

    /**
     * 角色登录成功, 服务器内部处理逻辑在此
     */
    public void onLogin() {
        logger.info("玩家登录成功...");

        this.refreshNewDay();

        for (ModuleManager manager : this.allManagers.values()) {
            try {
                manager.onPlayerEvent(PlayerEventType.AFTER_LOGIN);
            } catch (Exception e) {
                logger.error(manager.getClass().getName(), " AFTER_LOGIN 登录出错: ", this.getId(), ", ", e);
            }
        }
    }

    /**
     * 销毁玩家，释放资源
     */
    public void destroy() {
        this.doLogout();
        // 重置玩家数据
        this.data = new PlayerInfoPO();
        // 断开链接
        this.playerRemoveSessionBind();
        // 归还玩家对象
        GGame.playerPool.releasePlayer(this);
        logger.debug("Player {} has been destroyed.", this.getId());
    }

    /**
     * 每日刷新
     */
    public void refreshNewDay() {
        //0点重置数据
//        Date now = new Date();
//        if (DateUtils.isSameDay(now, this.memePunksData.refreshTime)) {
//            return;
//        }

        for (ModuleManager manager : this.allManagers.values()) {
            manager.onPlayerEvent(PlayerEventType.REFRESH_NEWDAY);
        }

        this.logger.info("refreshNewDay playerId : {}, name : {}", this.getId(), this.data.name);
    }

    @Override
    public void onLogoutUpdate() {
        this.logger.info("处理玩家下线逻辑 : {}, {}", this.getId(), this.data.name);
        // 下线相关模块处理
        for (ModuleManager manager : this.allManagers.values()) {
            manager.onPlayerEvent(PlayerEventType.OFFLINE);
        }
    }

    @Override
    public void sync() {
        this.logger.info("数据安全持久化...");
    }

    @Override
    public void free() {
        this.logger.info("释放玩家的相关引用...");
    }

    @Override
    public void playerRemoveSessionBind() {
        this.session.close();
        this.logger.info("玩家移除绑定了...");
    }

    @Override
    public void forbidPlayer(int forbidSec, String desc) {

    }

    @Override
    public int setLogoutTime(Date date) {
        return 0;
    }

    @Override
    public long getLogoutTime() {
        return 0;
    }

    @Override
    public void onPlayerUpdate() {

    }

    @Override
    public String getId() {
        return this.data.playerId;
    }

    @Override
    public int getLogicServerId() {
        return GGame.SERVER_ID;
    }

    @Override
    public long getUid() {
        return this.session.getUid();
    }

    @Override
    public void doUpdate() {

    }

}
